/**
 * NORRIS.js
 * Copyright (c) 2011 Ruben L Z Tan
 * 
 * Permission is hereby granted, free of charge, to any person obtaining a 
 * copy of this software and associated documentation files (the "Software"), 
 * to deal in the Software without restriction, including without limitation 
 * the rights to use, copy, modify, merge, publish, distribute, sublicense, 
 * and/or sell copies of the Software, and to permit persons to whom the 
 * Software is furnished to do so, subject to the following conditions:
 * 
 * The above copyright notice and this permission notice shall be included in 
 * all copies or substantial portions of the Software.
 * 
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR 
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, 
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE 
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER 
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING 
 * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS 
 * IN THE SOFTWARE.
 */
'use strict';

// Runs the CLI script
(function NORRIS() {
    
    // Make sure our paths work!
    process.chdir(__dirname);
    
    var colorize    = require(__dirname + '/colorize');
    var fs          = require('fs'); 
    
    var args        = process.argv;
    var options     = {};
    var commands    = [];
    
    // Ignore the first two argument
    for (var i = 2, len = args.length; i < len; i++) {
        var token = args[i];
        
        // Check for double dashed options (cannot be grouped)
        if ('--' === token.slice(0, 2)) {
            token = token.slice(2);
            options[token] = token;
        }
        // Check for single dashed options (can be grouped)
        else if ('-' === token.charAt(0)) {
            token = token.slice(1);
            if (token.length > 1) {
                for (var j = token.length; j--; ) {
                    options[token[j]] = token[j];
                }
            }
            else {
                options[token] = token;
            }
        }
        // Save the token for future reference
        else {
            commands.push(token);
        }
    }
    
    // Util object that makes life easier for other tasks
    var util = {
        inputData : null,
        matchData : null,
        
        input : function input(data) {
            this.inputData = data;
            return this;
        },
        
        options : function options(args) {
            this.matchData = args;
            return this;
        },
        
        check : function check() {
            var unsupported = [];
            for (var i in this.inputData) {
                var input = this.inputData[i];
                
                if (-1 === this.matchData.indexOf(input))
                    unsupported.push(input);
            }
            
            return unsupported;
        },
        
        has : function has() {
            var args = Array.prototype.slice.call(arguments);
            var keys = Object.keys(this.inputData);
            
            for (var i in args) {
                var a = args[i];
                if (-1 !== keys.indexOf(a))
                    return true;
            }
            
            return false;
        },
        
        reset : function reset() {
            this.inputData = this.matchData = null;
            return this;
        },
        
        log : function log(msg, nl) {
            if (undefined === nl) nl = true;
            
            process.stdout.write(msg);
            if (nl) process.stdout.write('\n');
        },
        
        color : function color(msg, nl) {
            if (undefined === nl) nl = true;
            
            process.stdout.write(colorize.ansify(msg));
            if (nl) process.stdout.write('\n');
        },
        
        help : function help(file) {
            var self = this;
            var path = __dirname + '/help/' + file + '.txt';
            
            fs.readFile(path, 'utf-8', function onRead(err, data) {
                if (!err) {
                    self.log(data);
                }
                else {
                    self.color('#red[Unable to find help file at ' + path + ']');
                }
            });
        }
    }
    
    // Get the first command token
    var task = commands.shift();
    switch (task) {
        case 'test'         : require(__dirname + '/test').run(util, commands, options);
            break;
            
        case 'version'      : require(__dirname + '/version').run(util);
            break;
            
        case 'help' :
        default : 
            require(__dirname + '/help').run(util);
            break;
    }
})();