/**
 * NORRIS.js
 * Copyright (c) 2011 Ruben L Z Tan
 * 
 * Permission is hereby granted, free of charge, to any person obtaining a 
 * copy of this software and associated documentation files (the "Software"), 
 * to deal in the Software without restriction, including without limitation 
 * the rights to use, copy, modify, merge, publish, distribute, sublicense, 
 * and/or sell copies of the Software, and to permit persons to whom the 
 * Software is furnished to do so, subject to the following conditions:
 * 
 * The above copyright notice and this permission notice shall be included in 
 * all copies or substantial portions of the Software.
 * 
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR 
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, 
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE 
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER 
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING 
 * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS 
 * IN THE SOFTWARE.
 */
'use strict';

// Required libraries
var NodeMock    = require('nodemock');
var MockRequest = require('mock-request');

var BaseMock = module.exports = {
    load : load
}

/**----------------------------------------------------------------------------
 * Loads a mock function to create a fixture environment for the unit tests
 */
function load(mock) {
    var mocked = {};
    
    // TODO : Optimize this!
    var path = mock.replace(/\.test/, '').replace(/\.js/, '');
    
    try {
        mocked = require(__dirname + '/' + path + '.mock');
    }
    catch (e) {
        console.log(e.stack);
        return null;
    }
        
    // General mocking modules
    mocked.mocker   = NodeMock;
    mocked.request  = MockRequest;
    
    // Creates the test -> function map
    mocked.make = function make(test) {
        // Convert the test (in dot notation) to a function signature (camel)
        var fn = test.replace(/(\.[a-z])/g, function($1){return $1.toUpperCase().replace('.','');});
        
        // If the function exists, call it and return the injection object
        if ('function' === typeof this[fn])
            return this[fn].call(this);
    }
    
    return mocked;
}
