/**
 * NORRIS.js
 * Copyright (c) 2011 Ruben L Z Tan
 * 
 * Permission is hereby granted, free of charge, to any person obtaining a 
 * copy of this software and associated documentation files (the "Software"), 
 * to deal in the Software without restriction, including without limitation 
 * the rights to use, copy, modify, merge, publish, distribute, sublicense, 
 * and/or sell copies of the Software, and to permit persons to whom the 
 * Software is furnished to do so, subject to the following conditions:
 * 
 * The above copyright notice and this permission notice shall be included in 
 * all copies or substantial portions of the Software.
 * 
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR 
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, 
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE 
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER 
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING 
 * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS 
 * IN THE SOFTWARE.
 */
'use strict';

module.exports.make = make;

/**----------------------------------------------------------------------------
 * Creates the test case
 */
function make(vows, assert, mocker) {
    var suite = vows.describe('Factory Unit Test');
    
    // Namespace loading test
    suite.addBatch({
        'GIVEN the mock environment "unit.factory.namespace" (see factory.mock) WHEN we add the namespace "lib" to norris' : {
            topic : function topic() {
                var factory = mocker.make('unit.factory.namespace');
                return factory.add('lib', 'norris');
            },
            
            'THEN the norris namespace should exists' : function testLvl0(norris) {
                assert.ok(false);
                assert.isNotNull(norris);
            },
            
            'THEN norris.lib should exists' : function testLvl1(norris) {
                assert.doesNotThrow(norris, Error);
                assert.isNotNull(norris.lib);
            },
            
            'THEN norris.lib.test1 and norris.lib.test2 should exist' : function testLvl2(norris) {
                assert.doesNotThrow(norris.lib, Error);
                assert.isNotNull(norris.lib.test1);
                assert.isNotNull(norris.lib.test2);
            },
            
            'THEN norris.lib.test2.test2a and norris.lib.test2.test2b' : function testLvl3(norris) {
                assert.doesNotThrow(norris.lib.test2, Error);
                assert.isNotNull(norris.lib.test2.test2a);
                assert.isNotNull(norris.lib.test2.test2b);
            }
        },
        
        'GIVEN that this is a second context' : {
            topic : function topic() {
                var factory = mocker.make('unit.factory.namespace');
                return factory.add('lib', 'norris');
            },
            
            'THEN I should be happy' : function happy() {
                assert.ok(true);
            },
            
            'GIVEN that this is a nested context' : {
                topic : function topic() {
                    return true;
                },
                
                'THEN I should be even happier' : function happier() {
                    assert.ok(true);
                }
            }
        }
    })
    
    
    
    return suite;
}
